//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizer;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;

public class DirectorySelectionActivity extends Activity {
	// Directory which this activity currently displays.
	private File currentDirectory;

	// An adapter for expansion listview.
	private ArrayAdapter<String> directoryListAdapter;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_directory_selection);

		//
		// Initialize members.
		//
		currentDirectory = null;
		directoryListAdapter = new ArrayAdapter<String>(this, 
				android.R.layout.simple_list_item_1);

		//
		// Set a listener for the directory list.
		//
		ListView directoryListView = (ListView)
				findViewById(R.id.directoryListView);
		directoryListView.setOnItemClickListener(
				new OnItemClickListener() {
					@Override
					public void onItemClick(AdapterView<?> parent,
							View view,int position, long ld) {
						if (position == 0)
							changeDirectoryToParent();
						else {
							changeDirectory(
								directoryListAdapter.getItem(position));
						}
					}
				});

		//
		// Set initial data.
		//
		loadDataDirectoryPath();

		//
		// Draw the activity.
		//
		drawActivity();
	}

	//
	// Handles an on-save-instance-state event of the activity.
	//
	@Override
	protected void onSaveInstanceState(Bundle state) {
		super.onSaveInstanceState(state);

		state.putString("current-directory",
				currentDirectory.getAbsolutePath());

		ListView lv = (ListView)findViewById(R.id.directoryListView);
		state.putInt("listview-position", lv.getFirstVisiblePosition());
	}

	//
	// Handles an on-save-instance-state event of the activity.
	//
	@Override
	protected void onRestoreInstanceState(Bundle state) {
		super.onRestoreInstanceState(state);

		String currentDirectoryPath = state.getString("current-directory");
		if (currentDirectoryPath != null)
			currentDirectory = new File(currentDirectoryPath);
		drawActivity();

		ListView lv = (ListView)findViewById(R.id.directoryListView);
		lv.setSelection(state.getInt("listview-position"));
	}

	//
	// Handle an on-create event of the option menu.
	//
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		return true;
	}

	//
	// Handle an on-options-item-selected event of the option menu.
	//
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		return true;
	}

	//
	// Determine path to a data directory.
	//
	private void loadDataDirectoryPath() {
		String dataDirectoryPath = null;
		File statusFileDir = getFilesDir();
		if (statusFileDir == null)
			Log.e("setDataDirectoryPath", "getFilesDir() returns null");
		else {
			DataDirectoryPathFileReader reader = null;
			try {
				File pathFile = new File(statusFileDir,
						getString(R.string.data_directory_path_file_name));
				reader = new DataDirectoryPathFileReader(pathFile);
				dataDirectoryPath = reader.getDataDirectoryPath();
			} catch (Exception e) {
				Log.e("setDataDirectoryPath", e.getMessage());
			}
		}

		if (dataDirectoryPath == null) {
			File externalStorageDirectory =
					Environment.getExternalStorageDirectory();
			if (externalStorageDirectory != null)
				dataDirectoryPath = externalStorageDirectory.getAbsolutePath();
		}

		if (dataDirectoryPath == null)
			dataDirectoryPath = File.pathSeparator;

		currentDirectory = new File(dataDirectoryPath);
	}

	//
	// Draw a screen.
	//
	private void drawActivity() {
		if (currentDirectory == null)
			return;

		//
		// Show the current directory path.
		//
		TextView currentDirectoryPathTextView = (TextView)
				findViewById(R.id.currentDirectoryPathTextView);
		currentDirectoryPathTextView.setText(
				currentDirectory.getAbsolutePath());

		//
		// Show sub-directories.
		//
		directoryListAdapter.clear();
		directoryListAdapter.add(getString(R.string.to_the_parent_folder));

		File[] files = null;
		try {
			files = currentDirectory.listFiles();
		} catch (Exception e) {
			; // Nothing to do.
		}

		if (files != null) {
			List<String> names = new ArrayList<String>();
			for (File file : files) {
				if (file.isDirectory() && !file.isHidden())
					names.add(file.getName());
			}
			Collections.sort(names, new Comparator<String>() {
				@Override
				public int compare(String s1, String s2) {
					return s1.compareToIgnoreCase(s2);
				}
			});
			for (String name : names)
				directoryListAdapter.add(name);
		}

		ListView directoryListView = (ListView)
				findViewById(R.id.directoryListView);
		directoryListView.setAdapter(directoryListAdapter);
	}

	private void changeDirectory(String directoryName) {
		currentDirectory = new File(currentDirectory, directoryName);
		drawActivity();
	}

	private void changeDirectoryToParent() {
		File parentDirectory = currentDirectory.getParentFile();
		if (parentDirectory != null && parentDirectory.canRead()) {
			currentDirectory = parentDirectory;
			drawActivity();
		}
	}

	//
	// Handle an on-click event of the "OK" button.
	//
	public void onOkButtonClick(View view) {
		Intent in = new Intent();
		if (currentDirectory != null)
			in.putExtra("path", currentDirectory.getAbsolutePath());
		setResult(RESULT_OK, in);
		finish();
	}
}
