//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizerlib;

/**
 * <p>
 * Represents a card of a deck building game.
 * </p>
 *
 * <p>
 * Its object memories ID, a title and a price of a card.  Note that type of
 * price is 'String' not 'int', since some games use '5+', '?' and so on.
 * It doesn't handles detailed explanation of a card (e.g. "+2 Actions").
 * </p>
 *
 * @author   Motoyuki Kasahara
 * @version  1.0
 */
public class Card implements Comparable<Card> {
	private String id;
	private String title;
	private String price;
	private Expansion expansion;

	/**
	 * Constructs an object.
	 *
	 * The created object has the specified ID, title, price and expansion
	 * which provides this card.
	 *
	 * @param  idArg          ID
	 * @param  titleArg       card title
	 * @param  priceArg       price
	 * @param  expansionArg   expansion
	 */
	public Card(String idArg, String titleArg, String priceArg,
		Expansion expansionArg) {
		assert idArg != null;
		assert titleArg != null;
		assert priceArg != null;
		assert expansionArg != null;

		id        = idArg;
		title     = titleArg;
		price     = priceArg;
		expansion = expansionArg;
	}

	/**
	 * Constructs a copy of the specified object.
	 *
	 * @param  card  the source object
	 */
	public Card(Card card) {
		assert card != null;

		id        = card.id;
		title     = card.title;
		price     = card.price;
		expansion = card.expansion;
	}

	/**
	 * Returns ID of this card.
	 *
	 * @return  ID
	 */
	public String getId() {
		return id;
	}

	/**
	 * Returns the title of this card.
	 *
	 * @return  a title
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * Returns price of this card.
	 *
	 * @return  price
	 */
	public String getPrice() {
		return price;
	}

	/**
	 * Returns an expansion which provides this card.
	 *
	 * @return  an expansion
	 */
	public Expansion getExpansion() {
		return expansion;
	}

	/**
	 * Returns true if this object is equal to the specified one.
	 *
	 * It returns true, if both objects have the same ID, 
	 *
	 * @param  o  an object to be compared
	 * @return  true if both objects have the same ID
	 */
	@Override
	public boolean equals(Object o) {
		if (o == this)
			return true;
		if (o == null || !(o instanceof Card))
			return false;
		Card card = (Card)o;
		return id.equals(card.id);
	}

	/**
	 * Returns a hash value of this object.
	 *
	 * @return  a hash value
	 */
	@Override
	public int hashCode() {
		if (id == null)
			return 0;
		else
			return id.hashCode();
	}

	/**
	 * Compares this object and the specified one.
	 * 
	 * It compares ID of both objects.  It returns an integer less than,
	 * equal to, or greater than zero if ID of this object is respectively
	 * less than, equal to, or greater than that of the specified object.
	 *
	 * @param  card  a card object to be compared
	 * @return  a negative integer, zero, or a positive integer according
	 *          with result of the comparison.
	 */
	public int compareTo(Card card) {
		return id.compareTo(card.id);
	}

	/**
	 * Converts this object to a String.
	 *
	 * @return  a title of this object.
	 */
	@Override
	public String toString() {
		return title;
	}
}
