//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizerlib;


/**
 * <p>
 * Represents an expansion of a deck building game (e.g. "Intrigue",
 * "Sea Side" etc. of "Dominion").  It also handles the base game as an
 * expansion.
 * </p>
 *
 * <p>
 * Its object has ID, a title and a preference of an expansion.  The
 * preference parameter is an integer value, used for sorting expansions by
 * publication date.  Its object doesn't manage what cards are provided by
 * the expansion.  For that purpose, use 'Game' class, instead.
 * </p>
 *
 * @author   Motoyuki Kasahara
 * @version  1.0
 */
public class Expansion implements Comparable<Expansion> {
	private String id;
	private String title;
	int preference;

	/**
	 * Constructs an object.
	 *
	 * The created object has the specified ID and title.  Its preference
	 * is set to the lowest value.
	 *
	 * @param  idArg     ID
	 * @param  titleArg  expansion title
	 */
	public Expansion(String idArg, String titleArg) {
		assert idArg != null;
		assert titleArg != null;

		id         = idArg;
		title      = titleArg;
		preference = Integer.MAX_VALUE;
	}

	/**
	 * Constructs an object.
	 *
	 * The created object has the specified ID, title and preference.
	 *
	 * @param  idArg          ID
	 * @param  titleArg       a title
	 * @param  preferenceArg  a preference value
	 */
	public Expansion(String idArg, String titleArg, int preferenceArg) {
		assert idArg != null;
		assert titleArg != null;

		id         = idArg;
		title      = titleArg;
		preference = preferenceArg;
	}

	/**
	 * Constructs a copy of the specified object.
	 *
	 * @param  expansion  the source object
	 */
	public Expansion(Expansion expansion) {
		assert expansion != null;

		id         = expansion.id;
		title      = expansion.title;
		preference = expansion.preference;
	}

	/**
	 * Returns ID of this object.
	 *
	 * @return  ID
	 */
	public String getId() {
		return id;
	}

	/**
	 * Returns a title of this expansion.
	 *
	 * @return  a title
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * Returns a preference of this expansion.
	 *
	 * @return  a preference value
	 */
	public int getPreference() {
		return preference;
	}

	/**
	 * Returns true if this object is equal to the specified one.
	 *
	 * It returns true, if both objects have the same ID, 
	 *
	 * @param  o  an object to be compared
	 * @return  true if both objects have the same ID
	 */
	@Override
	public boolean equals(Object o) {
		if (o == this)
			return true;
		if (o == null || !(o instanceof Expansion))
			return false;
		Expansion x = (Expansion)o;
		return id.equals(x.id);
	}

	/**
	 * Returns a hash value of this object.
	 *
	 * @return  a hash value
	 */
	@Override
	public int hashCode() {
		if (id == null)
			return 0;
		else
			return id.hashCode();
	}

	/**
	 * Compares this object and the specified one.
	 * 
	 * It compares ID of both objects.  It returns an integer less than,
	 * equal to, or greater than zero if ID of this object is respectively
	 * less than, equal to, or greater than that of the specified object.
	 *
	 * @param  expansion  an expansion object to be compared
	 * @return  a negative integer, zero, or a positive integer according
	 *          with result of the comparison.
	 */
	public int compareTo(Expansion expansion) {
		if (preference != expansion.preference)
			return preference - expansion.preference;
		else
			return id.compareTo(expansion.id);
	}

	/**
	 * Converts this object to a String.
	 *
	 * @return  a title of this object.
	 */
	@Override
	public String toString() {
		return title;
	}
}
