//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizerlib;

import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

/**
 * <p>
 * Represents a deck building game (e.g. "Dominion").
 * </p>
 *
 * <p>
 * Its object manages basic information about a game, expansions and cards.
 * Basic information of a game is set at construction of an object and it
 * cannot be changed.  Currently, basic information that an object hanles is
 * ID, a title, the number of cards to be selected at playing the game.
 * On the contrary, expansions and cards can be added and modified
 * successively.
 * </p>
 * 
 * @author   Motoyuki Kasahara
 * @version  1.0
 */
public class Game implements Comparable<Game> {
	private String id;
	private String title;
	private int selectionSize;

	private List<Expansion> expansions;
	private List<Card> cards;

	// cards also registered in 'cards', but its expansion is different.
	private List<Card> duplicateCards;

	/**
	 * Constructs an object.
	 *
	 * The created object has the specified ID, title and number of
	 * cards used at playing the game.
	 *
	 * @param  idArg              ID
	 * @param  titleArg           game title
	 * @param  selectionSizeArg   the number of selections
	 */
	public Game(String idArg, String titleArg, int selectionSizeArg) {
		assert idArg != null;
		assert titleArg != null;
		assert selectionSizeArg > 0;

		id            = idArg;
		title         = titleArg;
		selectionSize = selectionSizeArg;

		expansions     = new LinkedList<Expansion>();
		cards          = new LinkedList<Card>();
		duplicateCards = new LinkedList<Card>();
	}

	/**
	 * Constructs a copy of the specified object.
	 *
	 * @param  game  the source object
	 */
	public Game(Game game) {
		assert game != null;

		id            = game.id;
		title         = game.title;
		selectionSize = game.selectionSize;

		expansions     = new LinkedList<Expansion>(game.expansions);
		cards          = new LinkedList<Card>(game.cards);
		duplicateCards = new LinkedList<Card>(game.duplicateCards);
	}

	/**
	 * Returns ID of this game.
	 *
	 * @return  ID
	 */
	public String getId() {
		return id;
	}

	/**
	 * Returns a title of this game.
	 *
	 * @return  a title
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * Returns the number of selections.
	 *
	 * The term "the number of selections" means that the number of
	 * kinds of cards used at playing the game.
	 *
	 * @return  the number of selections
	 */
	public int getSelectionSize() {
		return selectionSize;
	}

	/**
	 * Removes all expansions and cards registered in this object.
	 */
	public synchronized void clear() {
		expansions.clear();
		cards.clear();
		duplicateCards.clear();
	}

	/**
	 * Returns the number of expansions registered in this object.
	 *
	 * @return  the number of expansions
	 */
	public int getExpansionsSize() {
		return expansions.size();
	}

	/**
	 * Returns a registered expansion at the specified position.
	 *
	 * @param  index  position of the expansion
	 * @return  an expansion
	 */
	public Expansion getExpansion(int index) {
		return expansions.get(index);
	}

	/**
	 * Returns a registered expansion with the specified ID.
	 *
	 * If no card matches, it returns null.
	 *
	 * @param   id  ID of the expansion
	 * @return  the found expansion, or null
	 */
	public Expansion getExpansion(String id) {
		assert id != null;

		ListIterator<Expansion> it = expansions.listIterator();
		while (it.hasNext()) {
			Expansion x = it.next();
			if (x.getId().equals(id))
				return x;
		}
		return null;
	}

	/**
	 * Returns a registered expansion, equal to the specified one.
	 *
	 * If no expansion matches, it returns null.
	 *
	 * @param   expansionArg  an expansion to be searched
	 * @return  the found expansion, or null
	 */
	public Expansion getExpansion(Expansion expansionArg) {
		assert expansionArg != null;

		ListIterator<Expansion> it = expansions.listIterator();
		while (it.hasNext()) {
			Expansion x = it.next();
			if (x.equals(expansionArg))
				return x;
		}
		return null;
	}

	/**
	 * Returns all registered expansions as a list.
	 *
	 * The expansions are sorted by their preference values.
	 *
	 * @return  a list of expansions
	 */
	public List<Expansion> getExpansions() {
		return new LinkedList<Expansion>(expansions);
	}

	/**
	 * Registers the specified expansion to this object.
	 *
	 * If the specified expansion has already been registered (i.e. they
	 * have the same ID), this method does nothing.
	 *
	 * @param  expansionArg  an expansion to be registered
	 * @return  true if the method registers the specified expansion
	 */
	public synchronized boolean addExpansion(Expansion expansionArg) {
		assert expansionArg != null;

		if (getExpansion(expansionArg) != null)
			return false;
		ListIterator<Expansion> it = expansions.listIterator();
		while (it.hasNext()) {
			Expansion x = it.next();
			if (x.preference > expansionArg.preference) {
				it.previous();
				break;
			}
		}
		it.add(expansionArg);
		return true;
	}

	/**
	 * Registers all the specified expansions to this object.
	 *
	 * If the specified expansion has already been registered (i.e. they have
	 * the same ID), this method skips it.
	 *
	 * @param  expansionsArg  a list of expansions to be registered
	 * @return  true if the method registers at least one expansion
	 */
	public synchronized boolean
	addExpansions(List<Expansion> expansionsArg) {
		assert expansionsArg != null;

		boolean changed = false;
		ListIterator<Expansion> it = expansionsArg.listIterator();
		while (it.hasNext()) {
			if (addExpansion(it.next()))
				changed = true;
		}
		return changed;
	}

	/**
	 * Removes the specified exansion from this object.
	 *
	 * If the specified expansion is registered (i.e. they have the same ID)
	 * in this object, this method removes it.  Otherwise, this method does
	 * nothing.
	 *
	 * This method also removes all cards provided by the specified expansion.
	 * If a card is provided by both the specified expansion and another one,
	 * the card provided by another one is still remained in this object.
	 *
	 * @param  expansionArg  an expansion to be removed
	 * @return  true if this method removes the expansion
	 */
	public synchronized boolean removeExpansion(Expansion expansionArg) {
		assert expansionArg != null;

		if (!expansions.remove(expansionArg))
			return false;

		ListIterator<Card> it = duplicateCards.listIterator();
		while (it.hasNext()) {
			Card c = it.next();
			if (!c.getExpansion().equals(expansionArg))
				continue;
			it.remove();
		}

		List<Card> cl = new LinkedList<Card>();
		it = cards.listIterator();
		while (it.hasNext()) {
			Card c = it.next();
			if (!c.getExpansion().equals(expansionArg))
				continue;
			it.remove();

			ListIterator<Card> it2 = duplicateCards.listIterator();
			while (it2.hasNext()) {
				Card c2 = it2.next();
				if (c2.equals(c)) {
					it2.remove();
					cl.add(c2);
					break;
				}
			}
		}
		cards.addAll(cl);

		return true;
	}

	/**
	 * Removes all the specified exansions from this object.
	 *
	 * If the specified expansion is registered (i.e. they have the same ID)
	 * in this object, this method removes it.  Otherwise, this method skips
	 * it.
	 *
	 * This method also removes all cards provided by the specified
	 * expansions.  If a card is provided by both the specified expansion and
	 * another one, the card provided by another one is still remained in
	 * this object.
	 *
	 * @param  expansionsArg  a list of expansions to be removed
	 * @return  true if this method removes at least one expansion
	 */
	public synchronized boolean
	removeExpansions(List<Expansion> expansionsArg) {
		assert expansionsArg != null;

		boolean changed = false;
		ListIterator<Expansion> it = expansionsArg.listIterator();
		while (it.hasNext()) {
			if (removeExpansion(it.next()))
				changed = true;
		}
		return changed;
	}

	/**
	 * Removes all exansions not specififed by the argument.
	 *
	 * If the specified expansion is registered (i.e. they have the same ID)
	 * in this object, this method remains it.  This method removes all
	 * unspecified expansions from this object.
	 *
	 * This method also removes all cards provided by unspecified expansions.
	 * If a card is provided by both unspecified and the specified expansions,
	 * the card provided by the specified one is still remained in this
	 * object.
	 *
	 * @param  expansionsArg  a list of expansions
	 * @return  true if this method removes at least one expansion
	 */
	public synchronized boolean
	removeOtherExpansions(List<Expansion> expansionsArg) {
		assert expansionsArg != null;

		List<Expansion> xl = new LinkedList<Expansion>();
		ListIterator<Expansion> it = expansions.listIterator();
		while (it.hasNext()) {
			Expansion x = it.next();
			if (expansionsArg.indexOf(x) < 0)
				xl.add(x);
		}
		return removeExpansions(xl);
	}

	/**
	 * Returns the number of registered cards.
	 *
	 * @return  the number of cards
	 */
	public int getCardsSize() {
		return cards.size();
	}

	/**
	 * Returns the number of registered cards of the specified expansion.
	 *
	 * @return  the number of cards
	 */
	public int getCardsSize(Expansion expansionArg) {
		int size = 0;

		if (!expansions.contains(expansionArg))
			return 0;

		ListIterator<Card> it = cards.listIterator();
		while (it.hasNext()) {
			if (it.next().getExpansion().equals(expansionArg))
				size++;
		}

		it = duplicateCards.listIterator();
		while (it.hasNext()) {
			if (it.next().getExpansion().equals(expansionArg))
				size++;
		}
		
		return size;
	}

	/**
	 * Returns a registered card at the specified position.
	 *
	 * @param  index  position of the card
	 * @return  a card
	 */
	public Card getCard(int index) {
		return cards.get(index);
	}

	/**
	 * Returns a registered card with the specified ID.
	 *
	 * If no card matches, it returns null.
	 *
	 * @param   id  ID of the card
	 * @return  the found card, or null
	 */
	public Card getCard(String id) {
		assert id != null;

		ListIterator<Card> it = cards.listIterator();
		while (it.hasNext()) {
			Card c = it.next();
			if (c.getId().equals(id))
				return c;
		}
		return null;
	}

	/**
	 * Returns a registered card, equal to the specified one.
	 *
	 * If no card matches, it returns null.
	 *
	 * @param   cardArg  a card to be searched
	 * @return  the found card, or null
	 */
	public Card getCard(Card cardArg) {
		assert cardArg != null;

		ListIterator<Card> it = cards.listIterator();
		while (it.hasNext()) {
			Card c = it.next();
			if (c.equals(cardArg))
				return c;
		}
		return null;
	}

	/**
	 * Returns all cards registered in this object as a list.
	 *
	 * @return  a list of cards
	 */
	public List<Card> getCards() {
		return new LinkedList<Card>(cards);
	}

	/**
	 * Returns all cards provided by the specified expansion.
	 * 
	 * This method searches registered cards and returns all cards provided
	 * by the specified expansion as a list form.
	 *
	 * It returns null if the specified expansion is not registered in this
	 * object.  It returns an empty list if the specified expansion is
	 * registered but no card of the expansion is registered.
	 *
	 * @param  expansionArg  an expansion
	 * @return  a list of cards, or null
	 */
	public List<Card> getCards(Expansion expansionArg) {
		assert expansionArg != null;

		if (!expansions.contains(expansionArg))
			return null;

		List<Card> cl = new LinkedList<Card>();
		ListIterator<Card> it = cards.listIterator();
		while (it.hasNext()) {
			Card c = it.next();
			if (c.getExpansion().equals(expansionArg))
				cl.add(c);
		}

		it = duplicateCards.listIterator();
		while (it.hasNext()) {
			Card c = it.next();
			if (c.getExpansion().equals(expansionArg))
				cl.add(c);
		}
		
		return cl;
	}

	/**
	 * Registers the specified card to this object.
	 *
	 * If the expansion of the card is not registered, this method also
	 * registers the expansion.
	 *
	 * If the specified card has already been registered (i.e. they have
	 * the same ID), this method doesn't register it again.  However, if it
	 * is provided by a different expansion, this method memories the
	 * specified card as "a duplicate card".  When an expansion of a
	 * registered card is removed, also the card is removed but corresponding
	 * duplicate cards, if exists, are still remained and one of them is
	 * registered instead.
	 *
	 * @param  cardArg  a card to be registered
	 * @return  true if this method registers a card or memories it as a
	 *          duplicate card.
	 */
	public synchronized boolean addCard(Card cardArg) {
		assert cardArg != null;

		Expansion x = cardArg.getExpansion();

		Card c = getCard(cardArg);
		if (c == null) {
			addExpansion(x);
			cards.add(cardArg);
			return true;
		} else if (!c.getExpansion().equals(x)) {
			boolean found = false;
			ListIterator<Card> it = duplicateCards.listIterator();
			while (it.hasNext()) {
				c = it.next();
				if (c.equals(cardArg) && c.getExpansion().equals(x)) {
					found = true;
					break;
				}
			}
			if (!found)
				duplicateCards.add(cardArg);
			return found;
		} else {
			return false;
		}
	}

	/**
	 * Registers all the specified cards to this object.
	 *
	 * If the expansion of the card is not registered, this method also
	 * registers the expansion.
	 *
	 * If the specified card has already been registered (i.e. they have
	 * the same ID), this method doesn't register it again.  However, if it
	 * is provided by a different expansion, this method memories the
	 * specified card as "a duplicate card".  When an expansion of a
	 * registered card is removed, also the card is removed but corresponding
	 * duplicate cards, if exists, are still remained and one of them is
	 * registered instead.
	 *
	 * @param  cardsArg  cards to be registered
	 * @return  true if this method registers at least one card or
	 *          memories at least one card as a duplicate card.
	 */
	public synchronized boolean addCards(List<Card> cardsArg) {
		assert cardsArg != null;

		boolean changed = false;
		ListIterator<Card> it = cardsArg.listIterator();
		while (it.hasNext()) {
			if (addCard(it.next()))
				changed = true;
		}
		return changed;
	}

	/**
	 * Removes the specified card from this object.
	 *
	 * If the specified card is registered (i.e. they have the same ID) in
	 * this object, this method removes the card.  Otherwise, this method
	 * does nothing.
	 *
	 * @param  cardArg  a card to be removed
	 * @return  true if this method removes a card
	 */
	public synchronized boolean removeCard(Card cardArg) {
		assert cardArg != null;

		boolean changed = false;
		ListIterator<Card> it = cards.listIterator();
		while (it.hasNext()) {
			Card c = it.next();
			if (c.equals(cardArg)) {
				it.remove();
				changed = true;
				break;
			}
		}

		it = duplicateCards.listIterator();
		while (it.hasNext()) {
			Card c = it.next();
			if (c.equals(cardArg)) {
				it.remove();
				changed = true;
			}
		}

		return changed;
	}

	/**
	 * Removes all the specified cards from this object.
	 *
	 * If the specified card is registered (i.e. they have the same ID) in
	 * this object, this method removes the card.  Otherwise, this method
	 * skips the card.
	 *
	 * @param  cardsArg  a list of cards to be removed
	 * @return  true if this method removes at least one card
	 */
	public synchronized boolean removeCards(List<Card> cardsArg) {
		assert cardsArg != null;

		boolean changed = false;
		ListIterator<Card> it = cardsArg.listIterator();
		while (it.hasNext()) {
			if (removeCard(it.next()))
				changed = true;
		}
		return changed;
	}

	/**
	 * Returns true if this object is equal the specified one.
	 *
	 * It returns true, if both objects have the same ID, 
	 *
	 * @param  o  an object to be compared
	 * @return  true if both objects have the same ID
	 */
	public boolean equals(Object o) {
		if (o == this)
			return true;
		if (o == null || !(o instanceof Game))
			return false;
		Game x = (Game)o;
		return id.equals(x.id);
	}

	/**
	 * Returns a hash value of this object.
	 *
	 * @return  a hash value
	 */
	@Override
	public int hashCode() {
		if (id == null)
			return 0;
		else
			return id.hashCode();
	}

	/**
	 * Compares this object and the specified one.
	 * 
	 * It compares ID of both objects.  It returns an integer less than,
	 * equal to, or greater than zero if ID of this object is respectively
	 * less than, equal to, or greater than that of the specified object.
	 *
	 * @param  game  a game object to be compared
	 * @return  a negative integer, zero, or a positive integer according
	 *          with result of the comparison.
	 */
	public int compareTo(Game game) {
		return id.compareTo(game.id);
	}

	/**
	 * Converts this object to a String.
	 *
	 * @return  a title of this object.
	 */
	@Override
	public String toString() {
		return title;
	}
}
