//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizerlib;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

/**
 * <p>
 * Builds 'Game' objects.
 * </p>
 *
 * <p>
 * Since 'GameBuilder' itself is an abstract class, use a concrete sub
 * class to build objects.
 * </p>
 *
 * @author   Motoyuki Kasahara
 * @version  1.0
 */
public abstract class GameBuilder {
	protected Locale locale;

	/**
	 * Constructs an object.
	 *
	 * The current value of the default locale is used for building
	 * a list of games.
	 */
	public GameBuilder() {
		locale = (Locale)Locale.getDefault().clone();
	}

	/**
	 * Constructs an object.
	 *
	 * The current value of the specified locale object is used for builing
	 * 'Game' objects.
	 *
	 * @param  localeArg  a locale object
	 */
	public GameBuilder(Locale localeArg) {
		assert localeArg != null;
		locale = (Locale)localeArg.clone();
	}

	/**
	 * Builds a 'Game' object with the specified ID.
	 *
	 * If game data about the specified ID are found, then this method
	 * constructs a new 'Game' object, puts the data into the object and
	 * returns it.  Otherwise, it returns null.
	 *
	 * @return  the built game object, or null
	 */
	public abstract Game build(String id)
		throws GameBuilderException;

	/**
	 * Builds the specified 'Game' object.
	 *
	 * If data about the specified game are found, this method registers
	 * the found expansions and cards to the specified game object using
	 * Game.AddExpansion() and Game.AddCard() methods.
	 *
	 * @param  game  a games to be built
	 * @return  the argument 'game'
	 */
	public abstract Game build(Game game)
		throws GameBuilderException;

	/**
	 * Builds one or more game objects.
	 *
	 * If game data are found, then this method constructs new 'Game'
	 * objects, puts the data into the objects and returns all of them as
	 * a list.  Otherwise, it returns null.
	 *
	 * @return  the built List&lt;Game&gt; object
	 */
	public List<Game> buildAll()
		throws GameBuilderException {
		return buildAll(new ArrayList<Game>());
	}

	/**
	 * Builds one or more game objects.
	 *
	 * If data about a game in 'gameList' are found, this method adds the
	 * found expansions and cards to the corresponding specified game objects
	 * using Game.AddExpansion(), Game.AddExpansions(), Game.AddCard() and
	 * Game.AddCards() methods.
	 *
	 * If data about a game not in 'gameList' are found, this method
	 * constructs a new 'Game' object, puts the data into the object and
	 * appends the object to 'gameList'.
	 *
	 * @param  gameList  a list of games
	 * @return  the argument 'gameList'
	 */
	public abstract List<Game> buildAll(List<Game> gameList)
		throws GameBuilderException;

	/**
	 * Converts this object to a String.
	 *
	 * @return  a string "CLASS-NAME for LOCALE"
	 */
	@Override
	public String toString() {
		return String.format("%s for %s",
			getClass().getSimpleName(), locale.toString());
	}
}
