//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizerlib;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

public class SimpleRandomizerCommand {
	enum OperationMode {
		RANDOMIZE, PRINT_GAMES, PRINT_EXPANSIONS, PRINT_CARDS, 
	}

	//
	// Main.
	// Usage: command [-d DIR] [-c|-g|-x] [-E] GAME [EXPANSION...]
	//
	public static void main (String[] args) {
		OperationMode operationMode = OperationMode.RANDOMIZE;
		CardComparisonMode comparisonMode
			= CardComparisonMode.PRICE_THEN_EXPANSION;

		ArrayList<String> argsList
			= new ArrayList<String>(Arrays.asList(args));
		String path = ".";

		//
		// Parse arguments.
		//
		while (!argsList.isEmpty()) {
			String arg = argsList.get(0);
			if (arg.equals("--")) {
				argsList.remove(0);
				break;
			} else if (arg.equals("-") || !arg.startsWith("-")) {
				break;
			}
			if (arg.equals("-c"))
				operationMode = OperationMode.PRINT_CARDS;
			else if (arg.equals("-E"))
				comparisonMode = CardComparisonMode.EXPANSION_THEN_PRICE;
			else if (arg.equals("-d")) {
				if (argsList.size() <= 1) {
					System.err.println("missing argument to '-d'");
					System.exit(1);
				}
				path = argsList.get(1);
				argsList.remove(0);
			} else if (arg.equals("-g")) {
				operationMode = OperationMode.PRINT_GAMES;
			} else if (arg.equals("-x")) {
				operationMode = OperationMode.PRINT_EXPANSIONS;
			} else {
				System.err.println(String.format("unknown option '%s'",
					argsList.get(0)));
				System.exit(1);
			}
			argsList.remove(0);
		}
		if (argsList.isEmpty() && operationMode != OperationMode.PRINT_GAMES) {
			System.err.println("no game ID specified");
			System.exit(1);
		}

		//
		// Load data of games.
		//
		List<Game> games = loadGames(path);
		if (games == null)
			System.exit(1);

		if (operationMode == OperationMode.PRINT_GAMES) {
			printGames(games);
			System.exit(0);
		}

		//
		// Choose game.
		//
		String gameId = argsList.get(0);
		argsList.remove(0);
		Game gameBase = selectGame(games, gameId);
		if (gameBase == null) {
			System.err.println("unknown game: " + gameId);
			System.exit(1);
		}

		//
		// Choose expansions.
		//
		Game game = selectExpansions(gameBase, argsList);
		if (game == null)
			System.exit(1);

		if (operationMode == OperationMode.PRINT_EXPANSIONS) {
			printExpansions(game.getExpansions());
			System.exit(0);
		} else if (operationMode == OperationMode.PRINT_CARDS) {
			printCards(game.getCards(), comparisonMode);
			System.exit(0);
		}

		List<Card> cards = selectCards(game);
		if (cards == null)
			System.exit(1);

		printCards(cards, comparisonMode);
	}

	//
	// Load data of games.
	//
	private static List<Game> loadGames(String path) {
		List<Game> games = null;
		try {            
			GameBuilder builder = new GameBuilderXMLFile(path);
			games = builder.buildAll();
		} catch (GameBuilderException e) {
			System.err.println(e.getMessage());
			return null;
		}
		if (games.size() == 0) {
			System.err.println("no game registered");
			return null;
		}
		return games;
	}

	//
	// Select a game to be used.
	//
	private static Game selectGame(List<Game> games, String id) {
		ListIterator<Game> it = games.listIterator();
		while (it.hasNext()) {
			Game g = it.next();
			if (g.getId().equals(id))
				return g;
		}
		return null;
	}

	//
	// Select expansions to be used.
	//
	private static Game selectExpansions(Game gameBase,
		List<String> ids) {
		Game game = new Game(gameBase);
		if (!ids.isEmpty()) {
			List<Expansion> xl = new ArrayList<Expansion>();
			ListIterator<String> it = ids.listIterator();
			while (it.hasNext()) {
				String id = it.next();
				Expansion x = game.getExpansion(id);
				if (x == null) {
					System.err.println("unknown expansion: " + id);
					return null;
				}
				xl.add(x);
			}
			game.removeOtherExpansions(xl);
		}

		return game;
	}

	//
	// Select cards randomly.
	//
	private static List<Card> selectCards(Game game) {
		Randomizer rnd = new Randomizer(game);
		List<Card> cards = new LinkedList<Card>();
		if (rnd.select(cards) < game.getSelectionSize())
			System.err.println("warning: not enough cards registered");
		return cards;
	}

	//
	// Print games.
	//
	private static void printGames(List<Game> games) {
		for (Game g : games) {
			String msg = String.format("%-20s  %-30s (%d)",
				g.getId(), g.getTitle(), g.getSelectionSize());
			System.out.println(msg);
		}
	}

	//
	// Print expansions.
	//
	private static void printExpansions(List<Expansion> expansions) {
		for (Expansion x : expansions) {
			String msg = String.format("%-20s  %-30s", x.getId(),
				x.getTitle());
			System.out.println(msg);
		}
	}

	//
	// Print cards.
	//
	private static void printCards(List<Card> cards, CardComparisonMode mode) {
		List<Card> cl = new ArrayList<Card>(cards);
		Collections.sort(cl, new CardComparator(mode));

		for (Card c : cl) {
			System.out.println(c.getTitle());
			System.out.println(String.format("    %-5s  %-30s",
				c.getPrice(), c.getExpansion().getTitle()));
		}
	}
}
