//
// Copyright (c) 2013  Motoyuki Kasahara
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
package jp.sourceforge.gamerandomizerlib;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import junit.framework.*;
import junit.textui.TestRunner;
import nedragtna.random.MTRandom;

public final class GameTest extends TestCase {
	public GameTest(String name) {
		super(name);
	}

	//
	// Test Game().
	//
	public void testCostructor() {
		try {
			String id = "test-game-id";
			String title = "test-game-title";
			int selectionSize = 10;

			Game g = new Game(id, title, selectionSize);

			assertEquals(g.getId(), id);
			assertEquals(g.getTitle(), title);
			assertEquals(g.getSelectionSize(), selectionSize);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test Game(game).
	//
	public void testCopyCostructor() {
		try {
			String id = "test-game-id";
			String title = "test-game-title";
			int selectionSize = 10;

			Game g1 = new Game(id, title, selectionSize);
			Game g2 = new Game(g1);

			assertEquals(g2.getId(), id);
			assertEquals(g2.getTitle(), title);
			assertEquals(g2.getSelectionSize(), selectionSize);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test getId(), getTitle() and getSelectionSize().
	//
	public void testBasicGetters() {
		try {
			String id = "test-game-id";
			String title = "test-game-title";
			int selectionSize = 10;

			int gamesSize = 10;
			Game[] games = new Game[gamesSize];
			for (int i = 0; i < gamesSize; i++)
				games[i] = new Game(id + i, title + i, selectionSize + i);

			for (int i = 0; i < gamesSize; i++) {
				assertEquals(games[i].getId(), id + i);
				assertEquals(games[i].getTitle(), title + i);
				assertEquals(games[i].getSelectionSize(), selectionSize + i);
			}
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test clear().
	//
	public void testClear() {
		try {
			Game g = new Game("test-game-id", "", 10);
			Expansion x1 = new Expansion("test-expansion-id1", "", 1000);
			Expansion x2 = new Expansion("test-expansion-id2", "", 1010);
			g.addExpansion(x1);
			g.addExpansion(x2);
			g.addCard(new Card("test-card-id1", "", "", x1));
			g.addCard(new Card("test-card-id1", "", "", x2));  // duplicate
			g.addCard(new Card("test-card-id2", "", "", x2));
			g.addCard(new Card("test-card-id3", "", "", x2));
			assertTrue(g.getExpansions().size() > 0);
			assertTrue(g.getCards().size() > 0);

			g.clear();
			
			assertTrue(g.getExpansions().size() == 0);
			assertTrue(g.getCards().size() == 0);

			//
			// Check if duplicate cards are also removed by clear().
			//
			g.addCard(new Card("test-card-id1", "", "", x1));
			g.removeCard(new Card("test-card-id1", "", "", x1));

			assertTrue(g.getCards().size() == 0);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Compare 'List<Expansion>' and 'Expansion ...expansions'.
	//
	public boolean compareExpansions(List<Expansion> list,
		Expansion ...expansions) {
		int i = 0;
		for (Expansion x : expansions) {
			if (list.size() >= i || !x.equals(list.get(i)))
				return false;
			i++;
		}
		return true;
	}
		
	//
	// Compare 'List<Card>' and 'Card ...cards'.
	//
	public boolean compareCards(List<Card> list, Card ...cards) {
		int i = 0;
		for (Card c : cards) {
			if (list.size() >= i || !c.equals(list.get(i)))
				return false;
			i++;
		}
		return true;
	}
		
	//
	// Test addExpansion().
	//
	public void testAddExpansion() {
		try {
			Game g = new Game("test-game-id", "", 10);

			Expansion x1 = new Expansion("test-expansion-id1", "", 1010);
			assertTrue(g.addExpansion(x1));
			compareExpansions(g.getExpansions(), x1);

			Expansion x2 = new Expansion("test-expansion-id2", "", 1020);
			assertTrue(g.addExpansion(x2));
			compareExpansions(g.getExpansions(), x1, x2);

			Expansion x3 = new Expansion("test-expansion-id3", "", 1000);
			assertTrue(g.addExpansion(x3));
			compareExpansions(g.getExpansions(), x3, x1, x2);

			Expansion x4 = new Expansion("test-expansion-id3", "?", 1030);
			assertFalse(g.addExpansion(x4));
			compareExpansions(g.getExpansions(), x3, x1, x2);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test addExpansions().
	//
	public void testAddExpansions() {
		try {
			Game g = new Game("test-game-id", "", 10);

			List<Expansion> xl = new ArrayList<Expansion>();
			Expansion x1 = new Expansion("test-expansion-id1", "", 1010);
			Expansion x2 = new Expansion("test-expansion-id2", "", 1020);
			Expansion x3 = new Expansion("test-expansion-id3", "", 1000);

			assertTrue(g.addExpansion(x1));
			compareExpansions(g.getExpansions(), x1);

			assertTrue(g.addExpansion(x2));
			compareExpansions(g.getExpansions(), x1, x2);

			assertTrue(g.addExpansion(x3));
			compareExpansions(g.getExpansions(), x3, x1, x2);

			Expansion x4 = new Expansion("test-expansion-id3", "?", 1030);
			assertFalse(g.addExpansion(x4));
			compareExpansions(g.getExpansions(), x3, x1, x2);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	//
	// Test toString().
	//
	public void testToString() {
		try {
			String title = "test-game-title";
			Game g = new Game("test-game-id", title, 10);
			assertEquals(g.toString(), title);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(GameTest.class);
	}
}
